require "bundler/gem_helper"

base_dir = File.join(__dir__, "..")
helper = Bundler::GemHelper.new(base_dir)
helper.install

release_task = Rake.application["release"]
# We use Trusted Publishing.
release_task.prerequisites.delete("build")
release_task.prerequisites.delete("release:rubygem_push")
release_task_comment = release_task.comment
if release_task_comment
  release_task.clear_comments
  release_task.comment = release_task_comment.gsub(/ and build.*$/, "")
end
